# -*- coding: utf-8 -*-
from pyramid.config import Configurator
from pyramid_beaker import session_factory_from_settings, set_cache_regions_from_settings
from zope.component import getGlobalSiteManager
from pyramid.authentication import RepozeWho1AuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid_skins.renderer import renderer_factory
import gevent_psycopg2; gevent_psycopg2.monkey_patch()

PROJECT_ID_BLACKLIST = ('portale', 'project', 'support', 'assistenza')

try:
    import pyinotify; pyinotify
    DISCOVERY = True
except ImportError:
    DISCOVERY = False


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    settings['mail.port'] = settings.get('mail_port', None) and int(settings['mail.port']) or 25
    globalreg = getGlobalSiteManager()
    set_cache_regions_from_settings(settings)
    config = Configurator(registry=globalreg)

    config.include('pyramid_zcml')
    config.load_zcml('workflow.zcml')
    config.add_translation_dirs('penelope.core:locale')

    from penelope.core.views import PORRequest
    config.setup_registry(settings=settings,
                          request_factory=PORRequest,
                          root_factory='penelope.core.views.DefaultContext')

    # por security
    from penelope.core import security
    authentication_policy = RepozeWho1AuthenticationPolicy(identifier_name="auth_tkt",callback=security.rolefinder)
    config._set_authentication_policy(authentication_policy)
    authorization_policy = ACLAuthorizationPolicy()
    config._set_authorization_policy(authorization_policy)
    config.scan('penelope.core.security.views')
    config.include('penelope.core.security.openid2')
    config.include('velruse.store.memstore')
    config.add_view('penelope.core.security.views.forbidden', renderer='skin', context="pyramid.httpexceptions.HTTPForbidden")

    #mailier
    config.include('pyramid_mailer')

    # penelope.core.models's configuration
    config.include('penelope.core.models')
    import penelope.core.events; penelope.core.events
    import penelope.core.breadcrumbs; penelope.core.breadcrumbs
    import penelope.core.sidebar; penelope.core.sidebar

    session_factory = session_factory_from_settings(settings)
    config.set_session_factory(session_factory)

    config.add_static_view('static', 'penelope.core:static')
    config.scan('penelope.core.views')
    config.add_route('socketio', 'socket.io/*remaining')
    config.scan('penelope.core.socketspace')

    config.add_route('tp', '/tp/*traverse', factory='penelope.core.tp.TPContext')
    config.scan('penelope.core.tp')
    config.scan('penelope.core.smartadd')
    config.scan('penelope.core.backlog')

    config.add_route('administrator', '/manage', factory='penelope.core.manage.ManageContext')
    config.add_route('manage_svn_authz', '/manage/svn_authz', factory='penelope.core.manage.ManageContext')
    config.add_route('manage_costs', '/manage/costs', factory='penelope.core.manage.ManageContext')
    config.scan('penelope.core.manage')

    config.add_route('search', '/search')
    config.scan('penelope.core.search')

    config.add_route('reports', '/reports/*traverse', factory='penelope.core.reports.ReportContext')
    config.scan('penelope.core.reports')
    config.add_renderer(name='xls_report', factory='penelope.core.renderers.XLSReportRenderer')
    config.add_renderer(name='csv_report', factory='penelope.core.renderers.CSVReportRenderer')

    config.include('pyramid_rpc.jsonrpc')
    config.add_jsonrpc_endpoint('DashboardAPI', '/apis/json/dashboard')
    config.scan('penelope.core.api')

    config.add_renderer('skin', renderer_factory)
    config.include('pyramid_skins')
    config.register_path("penelope.core:skins", discovery=DISCOVERY)

    # penelope.core.gdata configuration
    config.include('penelope.core.gdata')

    # pyramid_formalchemy's configuration
    config.include('pyramid_formalchemy')
    config.include('fa.bootstrap')
    config.include('deform_bootstrap')

    config.formalchemy_admin('admin',
                             package='penelope.core',
                             factory='penelope.core.forms.CrudModels',
                             models='penelope.core.models',
                             view='penelope.core.forms.ModelView',
                             session_factory='penelope.core.models.DBSession')

    config.add_view(context='pyramid_formalchemy.resources.ModelListing',
                    renderer='fa.bootstrap:templates/admin/new.pt',
                    request_method='POST',
                    route_name='admin',
                    request_type='penelope.core.interfaces.IPorRequest',
                    view='penelope.core.forms.security_create')

    config.add_static_view('deform_static', 'deform:static')
    config.add_route('navbar', '/navbar')
    config.add_route('favicon', '/favicon.ico')
    config.add_route('inbound_email', '/inbound_email')
    config.scan('penelope.core.notifications')

    from penelope.core.forms import include_forms
    include_forms(config)

    return config.make_wsgi_app()



# -*- coding: utf-8 -*-

import colander
import collections
import datetime
import itertools
import logging
import operator
import sqlalchemy as sa

from colander import SchemaNode
from deform import ValidationFailure
from deform.widget import SelectWidget
from pyramid.renderers import render
from pyramid.view import view_config
from repoze.workflow import get_workflow
from sqlalchemy.orm import lazyload
from sqlalchemy import distinct
from webhelpers.html.builder import HTML, literal

from penelope.core.events import AfterEntryCreatedEvent
from penelope.core.lib.helpers import ticket_url, timeentry_url, ReversedOrder, total_seconds
from penelope.core.lib.widgets import SearchButton, PorInlineForm
from penelope.core.reports import fields
from penelope.core.reports.favourites import render_saved_query_form
from penelope.core.reports.queries import qry_active_projects, te_filter_by_customer_requests, filter_users_with_timeentries, te_filter_by_contracts
from penelope.core.reports.validators import validate_period

from penelope.core.models import DBSession, Project, TimeEntry, User, CustomerRequest, Contract, Customer
from penelope.core.models.tickets import ticket_store
from penelope.core.models.tp import timedelta_as_human_str

log = logging.getLogger(__name__)


class CustomerReport(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request


    class CustomSchema(colander.MappingSchema):
        customer_id = fields.customer_id.clone()
        customer_id.missing = colander.null

        project_id = fields.project_id.clone()
        customer_requests = fields.customer_requests.clone()
        contracts = fields.contracts.clone()
        date_from = fields.date_from.clone()
        date_to = fields.date_to.clone()
        users = fields.users.clone()
        workflow_states = fields.workflow_states.clone()

        detail_level = SchemaNode(colander.String(),
                                  validator=colander.OneOf(['project', 'request', 'ticket', 'timeentry', 'date']),
                                  widget=SelectWidget(values=[
                                          ('project', 'Project'),
                                          ('request', 'Request'),
                                          ('ticket', 'Ticket'),
                                          ('timeentry', 'Time Entry'),
                                          ('date', 'Date'),
                                      ]),
                                  default='timeentry',
                                  missing=colander.null,
                                  title=u'Detail')

    def format_web(self, value):
        if isinstance(value, datetime.timedelta):
            return timedelta_as_human_str(value)
        elif isinstance(value, float):
            return u'%.2f €' % value
        return value


    def format_xls(self, value):
        if isinstance(value, datetime.timedelta):
            # Render durations as float in order to sum them
            return total_seconds(value) / 60.0 / 60.0
        if isinstance(value, literal):
            # sanitize HTML
            return value.striptags()
        return value


    def search(self, customer_id, project_id, date_from, date_to, contracts,
               users, customer_requests, workflow_states, detail_level, render_links=True):

        # also search archived projects, if none are specified
        qry = DBSession.query(TimeEntry).join(TimeEntry.project).join(Project.customer).outerjoin(TimeEntry.author)
        qry = qry.options(lazyload(TimeEntry.project, TimeEntry.author, Project.customer))

        if customer_id is not colander.null:
            qry = qry.filter(Project.customer_id == customer_id)

        if project_id is not colander.null:
            qry = qry.filter(TimeEntry.project_id == project_id)

        if date_from is not colander.null:
            qry = qry.filter(TimeEntry.date>=date_from)

        if date_to is not colander.null:
            qry = qry.filter(TimeEntry.date<=date_to)

        if users:
            qry = qry.filter(TimeEntry.author_id.in_(users))

        if workflow_states:
            qry = qry.filter(TimeEntry.workflow_state.in_(workflow_states))

        qry = qry.filter(te_filter_by_customer_requests(customer_requests, request=self.request))

        qry = qry.filter(te_filter_by_contracts(contracts))

        qry = qry.order_by(sa.desc(TimeEntry.project_id),
                           sa.desc(TimeEntry.ticket),
                           sa.desc(TimeEntry.date))

        time_entries = self.request.filter_viewables(qry)

        proj_tickets = collections.defaultdict(set)
        for te in time_entries:
            if te.ticket is None:
                continue
            proj_tickets[te.project_id].add(te.ticket)

        projectsmap = {}
        for project_id, ticket_ids in proj_tickets.items():
            project = DBSession.query(Project).get(project_id)
            projectsmap[project_id] = dict(ticket_store.get_requests_from_tickets(
                                            project, tuple(ticket_ids)))

        tkts = {}
        for project_id, ticket_ids in proj_tickets.items():
            project = DBSession.query(Project).get(project_id)
            for tkt in ticket_store.get_tickets_for_project(project):
                tkts[(project_id, tkt['id'])] = tkt

        rows = []

        for te in time_entries:
            customer_request = te.customer_request
            if render_links:
                rendered_request = HTML.A(customer_request.name.strip(),
                                            href="%s/admin/CustomerRequest/%s" % (
                                                self.request.application_url,
                                                customer_request.id
                                                )
                                            )
            else:
                rendered_request = customer_request.name.strip()

            tkt = tkts[(te.project_id, te.ticket)]

            ticket_type = tkt['type']
            ticket_summary = tkt['summary']
            if render_links:
                ticket_summary = HTML.A(ticket_summary,
                                        href=ticket_url(request=self.request,
                                                        project=te.project,
                                                        ticket_id=te.ticket))

            description = HTML.A(te.description,
                                 href=timeentry_url(request=self.request,
                                                    time_entry=te))

            entry = {
                'customer': te.project.customer.name.strip(),
                'project': te.project.name.strip(),
                'request': rendered_request,
                'ticket_summary': ticket_summary,
                'user': te.author.fullname.strip(),
                'date': te.date.strftime('%Y-%m-%d'),
                'description': description,
                'location': te.location,
                'ticket_type': ticket_type,
                'sensitive': ['no', 'yes'][tkt['sensitive']],
                'ticket_number': te.ticket,
                'ticket_url': ticket_url(request=self.request,
                                         project=te.project,
                                         ticket_id=te.ticket),
                'ticket_status': tkt['status'],
                'hours': te.hours,
            }

            event = AfterEntryCreatedEvent(entry, te)
            self.request.registry.notify(event)

            rows.append(entry)

        columns = [
            ('customer', u'Cliente'),
            ('project', u'Progetto'),
            ('request', u'Request'),
            ('ticket_summary', u'Ticket'),
            ('ticket_type', u'Tipologia'),
            ('sensitive', u'Sensitive'),
            ('user', u'Persona'),
            ('date', u'Data'),
            ('description', u'Descrizione attività'),
            ('location', u'Sede'),
            ('hours', u'Ore'),
        ]

        group_by = {
            'project': ['customer', 'project', 'user'],
            'request': ['customer', 'project', 'request', 'user'],
            'ticket': ['customer', 'project', 'request', 'ticket_summary',
                       'ticket_type', 'sensitive', 'user'],
            'timeentry': None,
            'date': ['user', 'date', 'location'],
        }[detail_level]

        if group_by:
            rows, columns = self.group(rows, columns, group_by)

        return {
            'rows': rows,
            'columns': columns,
        }

    def group(self, rows, columns, group_by):
        missing = set(group_by) - set(c[0] for c in columns)
        if missing:
            raise ValueError(u"Could not find column: %s" % ', '.join(missing))

        hours = collections.defaultdict(datetime.timedelta)

        for key, group in itertools.groupby(rows,
                                            operator.itemgetter(*group_by)):
            for row in group:
                hours[key] += row['hours']

        rows = [
                dict(zip(group_by, key), hours=hours[key])
                for key in hours
                ]

        # order by the grouped columns again
        rows.sort(
                key = lambda row: [
                    # date is sorted descedent
                    (ReversedOrder(row[k]) if k=='date' else row[k])
                    for k in group_by
                    ]
                )

        return rows, [c for c in columns if c[0] in (group_by + ['hours'])]

    @view_config(name='custom_json', route_name='reports', renderer='json', permission='reports_custom')
    def custom_json(self):
        schema = self.CustomSchema(validator=validate_period).clone()
        form = PorInlineForm(schema,
                             action=self.request.current_route_url())
        controls = self.request.GET.items()
        appstruct = form.validate(controls)
        detail = self.search(render_links=False, **appstruct)
        result = []
        for row in detail['rows']:
            row['hours'] = timedelta_as_human_str(row['hours'])
            if 'description' in row:
                del row['description']
            result.append(row)
        return result

    @view_config(name='custom_xls', route_name='reports', renderer='xls_report', permission='reports_custom')
    def custom_xls(self):
        schema = self.CustomSchema(validator=validate_period).clone()
        form = PorInlineForm(schema,
                             action=self.request.current_route_url())
        controls = self.request.GET.items()
        appstruct = form.validate(controls)

        detail = self.search(render_links=False, **appstruct)
        columns = detail['columns']

        rows = [
            [
                self.format_xls(row[col_key])
                for col_key, col_title in columns
            ] + [timedelta_as_human_str(row['hours']),
                 row['ticket_number'], row['ticket_url'], row['ticket_status']]
            for row in detail['rows']
        ]

        return {
            'header': [col_title for col_key, col_title in columns] +
                      ['Ore', 'Ticket #', 'Ticket URL', 'Ticket status'],
            'rows': rows,
        }


    @view_config(name='report_custom', route_name='reports', renderer='skin', permission='reports_custom')
    def __call__(self):
        schema = self.CustomSchema(validator=validate_period).clone()
        projects = self.request.filter_viewables(qry_active_projects())
        project_ids = [p.id for p in projects]
        customers = DBSession.query(distinct(Customer.id), Customer.id, Customer.name).join(Project).filter(Project.id.in_(project_ids)).order_by(Customer.name)
        users = DBSession.query(User).order_by(User.fullname)
        users = filter_users_with_timeentries(users)
        customer_requests = DBSession.query(CustomerRequest.id, CustomerRequest.name).order_by(CustomerRequest.name)
        contracts = DBSession.query(Contract.id, Contract.name).order_by(Contract.name)

        form = PorInlineForm(schema,
                             action=self.request.current_route_url(),
                             formid='report-customer',
                             method='GET',
                             buttons=[
                                 SearchButton(title=u'Search'),
                             ])

        workflow = get_workflow(TimeEntry(), 'TimeEntry')

        all_wf_states = [
                            (state, workflow._state_data[state]['title'] or state)
                            for state in workflow._state_order
                        ]

        form['workflow_states'].widget.values = all_wf_states
        # XXX the following validator is broken
        form['workflow_states'].validator = colander.OneOf([str(ws[0]) for ws in all_wf_states])

        form['customer_id'].widget.values = [('', '')] + [(str(c.id), c.name) for c in customers]
        # don't validate as it might be an archived customer
        form['project_id'].widget.values = [('', '')] + [(str(p.id), p.name) for p in projects]
        # don't validate as it might be an archived project

        form['users'].widget.values = [(str(u.id), u.fullname) for u in users]
        form['customer_requests'].widget.values = [(str(c.id), c.name) for c in customer_requests]
        form['contracts'].widget.values = [(str(c.id), c.name) for c in contracts]

        controls = self.request.GET.items()

        if not controls or len(controls)==1: # detail_level param
            # the form is empty
            return {
                    'form': form.render(),
                    'saved_query_form': render_saved_query_form(self.request),
                    'qs':'',
                    'result_table': None
                    }

        try:
            appstruct = form.validate(controls)
        except ValidationFailure as e:
            return {
                    'form': e.render(),
                    'saved_query_form': render_saved_query_form(self.request),
                    'qs':'',
                    'result_table': None
                    }

        detail = self.search(render_links=True, **appstruct)

        result_table = None

        if detail['rows']:
            base_link = self.request.path_url.rsplit('/', 1)[0]
            xls_link = ''.join([base_link, '/', 'custom_xls', '?', self.request.query_string])
            json_link = ''.join([base_link, '/', 'custom_json', '?', self.request.query_string])
            delta0 = datetime.timedelta()
            delta_tot = sum((row['hours'] for row in detail['rows']), delta0)
            human_tot = timedelta_as_human_str(delta_tot)

            result_table = render('penelope.core:reports/templates/custom_results.pt',
                                  {
                                      'rows': detail['rows'],
                                      'columns': detail['columns'],
                                      'xls_link': xls_link,
                                      'json_link': json_link,
                                      'format_web': self.format_web,
                                      'human_tot': human_tot,
                                  },
                                  request=self.request)

        return {
                'form': form.render(appstruct=appstruct),
                'saved_query_form': render_saved_query_form(self.request),
                'qs': self.request.query_string,
                'result_table': result_table,
                }





from sqlalchemy import Column, DateTime, Integer, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.orm import deferred
from sqlalchemy.ext.declarative import declared_attr
from datetime import datetime
from zope.interface import implements

from penelope.core.models.interfaces import IDublinCore
from pyramid.threadlocal import get_current_request


class DublinCore(object):
    """
    Base dublincore implementation for POR project
    """

    implements(IDublinCore)

    @declared_attr
    def creation_date(cls):
        return deferred(Column(DateTime), group='dublincore')

    @declared_attr
    def modification_date(cls):
        return deferred(Column(DateTime), group='dublincore')

    @declared_attr
    def author_id(cls):
        return deferred(Column(Integer, ForeignKey('users.id')), group='dublincore')

    @declared_attr
    def author(cls):
        return relationship('User', uselist=False, primaryjoin="%s.author_id==User.id" % cls.__name__)


def dublincore_insert(mapper, connection, target):
    target.creation_date = datetime.now()
    target.modification_date = datetime.now()

    #try to get author in gracemode:
    if hasattr(target, 'request'):
        request = target.request
    else:
        request = get_current_request()

    if request and not target.author_id:
        environ = request.environ
        if environ.has_key('repoze.who.identity'):
            if environ['repoze.who.identity'].has_key('user'):
                target.author_id = environ['repoze.who.identity']['user'].id


def dublincore_update(mapper, connection, target):
    target.modification_date = datetime.now()



import os
import robotsuite
import unittest

from webtest import http
from paste.deploy import loadapp
from pyramid_robot.layer import Layer, layered


class myPyramidLayer(Layer):

    defaultBases = ()

    def setUp(self):
        conf_dir = os.path.dirname(__file__)
        app = loadapp('config:test.ini', relative_to=conf_dir)
        from penelope.core.scripts import populate_dummies
        populate_dummies()
        self.server = http.StopableWSGIServer.create(app, port=8080)

    def tearDown(self):
        self.server.shutdown()

PYRAMIDROBOTLAYER = myPyramidLayer()


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [
        os.path.join('robot', doc) for doc in
        os.listdir(robot_dir) if doc.endswith('.robot') and
        doc.startswith('test_')
    ]
    for test in robot_tests:
        suite.addTests([
            layered(robotsuite.RobotTestSuite(test),
            layer=PYRAMIDROBOTLAYER),
        ])

    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



# -*- coding: utf-8 -*-
import itertools
import os
import re

from zope.interface import implements
from pyramid.response import Response
from pyramid import decorator
from pyramid.request import Request
from pyramid.i18n import TranslationStringFactory
from pyramid.view import view_config
from pyramid.security import has_permission

from penelope.core import fanstatic_resources, messages
from penelope.core.interfaces import IBreadcrumbs, IPorRequest, ISidebar
from penelope.core.lib.htmlhelpers import get_application_link
from penelope.core.security import acl
from penelope.core.lib.helpers import chunks, unicodelower
from penelope.core.models import DBSession, Project, KanbanBoard
from penelope.core.models.interfaces import IProjectRelated


_ = TranslationStringFactory('por')


class PORRequest(Request):

    model_instance = None
    challenge_item = None
    charset = 'utf8'

    implements(IPorRequest)

    def __init__(self, *args, **kwargs):
        super(PORRequest, self).__init__(*args, **kwargs)
        self.charset = 'UTF-8'

    def _get_message_from_session(self):
        msgs = self.session.get(messages.STATUSMESSAGEKEY)
        if not msgs:
            msgs = messages.Messages()
            self.session[messages.STATUSMESSAGEKEY] = msgs
        return msgs

    def add_message(self, *args, **kwargs):
        """\
        Call this once or more to add messages to be displayed at the
        top of the web page.

        Examples:

        >>> request.add_message(u'A random warning message', 'warning')

        If no type is given it defaults to 'info'
        >>> request.add_message(u'A random info message')

        The arguments are:
            request:   instance of pyramid Request
            message:   a string, with the text message you want to show,
                        or a HTML fragment
            type:      optional, defaults to 'info'. The type determines how
                        the message will be rendered, as it is used to select
                        the CSS class for the message. Predefined types are:
                        'success' - for successful actions
                        'info' - for informational messages
                        'warning' - for warning messages
                        'error' - for messages about restricted access or errors.
        """
        msgs = self._get_message_from_session()
        msgs.add(*args, **kwargs)

    def show_messages(self):
        msgs = self._get_message_from_session()
        value = msgs.show()
        if self.response.status_int not in (301, 302, 304):
            del self.session[messages.STATUSMESSAGEKEY]
        return value

    @decorator.reify
    def title(self):
        return self.root.title

    @decorator.reify
    def authenticated_user(self):
        user = None
        identity = self.environ.get('repoze.who.identity', None)
        if identity:
            user = identity.get('user', None)
        return user

    def isAuthenticated(self):
        return self.authenticated_user is not None

    def context_for_breadcrumbs(self, view):
        model = self.model_instance
        if model:
            return model
        else:
            return view

    def render_breadcrumbs(self, view):
        context = self.context_for_breadcrumbs(view)
        context.request = view.request
        breadcrumbs = IBreadcrumbs(context)
        return breadcrumbs.render(self)

    def render_sidebar(self, view):
        context = self.context_for_breadcrumbs(view)
        context.request = view.request
        sidebar = ISidebar(context)
        return sidebar.render(self)

    def fa_parent_url(self, context=None):
        if IProjectRelated.providedBy(self.model_instance):
            return self.fa_url('Project', self.model_instance.project.id)
        else:
            return self.fa_url(self.model_name)

    def active_topbar(self, paths):
        for path in paths:
            if path in self.path:
                return 'active'
        return ''

    def filter_viewables(self, items):
        if len([a for a in self.authenticated_user.roles if a.name == 'administrator']) == 1:
            # for administrator let's skip check 
            return items
        return [item for item in items if self.filter_viewable(item)]

    def filter_viewable(self, item):
        self.challenge_item = item
        if has_permission('view', item, self):
            self.challenge_item = None
            return item
        self.challenge_item = None

    def has_permission(self, permission, context):
        if not context:
            context = DefaultContext(self) #we always need a context for ACL to work
        self.challenge_item = context
        result = has_permission(permission, context, self)
        self.challenge_item = None
        return result


class DefaultContext(object):
    """
    Default context factory for TP views.
    """
    __acl__ = acl.DEFAULT_ACL

    def __init__(self, request):
        fanstatic_resources.dashboard.need()
        self.request = request

    def prettify_title(self, title):
        title = title.replace('_', ' ')
        return title

    @property
    def title(self):
        view = self.request.view_name or self.request.environ['PATH_INFO'].split('/')[-1]
        if view == '':
            title = 'Dashboard'
        elif view == 'add_entry':
            title = 'Smart add time entries'
        else:
            title = self.prettify_title(view).title()
        if not title:
            title = 'Penelope'
        return title


@view_config(context='pyramid.exceptions.NotFound', renderer='penelope.core:templates/notfound.pt')
def notfound_view(request):
    request.response.status_int = 404
    return {}


def group_by_customer(projects):
    return  [
                (x[0], list(x[1]))
                for x in itertools.groupby(projects, lambda x: x.customer)
            ]

@view_config(permission='view_home', route_name="favicon")
def favicon(request):
    here = os.path.dirname(__file__)
    icon = open(os.path.join(here, 'static', 'penelope', 'images', 'favicon.png'))
    return Response(content_type='image/x-icon', app_iter=icon)


@view_config(renderer='skin', permission='view_home')
def view_home(request):
    """
    Default home view
    """
    fanstatic_resources.dashboard_home.need()

    session = DBSession()
    user = request.authenticated_user
    projects = session.query(Project)

    my_projects = projects\
                  .filter(Project.users_favorite(user))\
                  .order_by(Project.customer_id).all()

    my_projects = request.filter_viewables(my_projects)

    active_projects = set(projects.filter(Project.active))
    active_projects = sorted(request.filter_viewables(active_projects.difference(my_projects)), key=unicodelower)

    listings = []
    listing_columns = 4
    listings.append({'title': 'Active projects',
                     'projgroups': tuple(chunks(tuple(active_projects), listing_columns)),})
    kanbanboards = user.kanban_boards

    return {
            'my_projects': my_projects,
            'kanbanboards': kanbanboards,
            'listings': listings,
            'can_see_kanbanboards':  request.has_permission('listing', KanbanBoard),
            'get_application_link': get_application_link,
            }


@view_config(renderer='skin', permission='view_home', route_name='navbar')
def view_navbar(request):
    """
    Navbar is rendered separately, and served to Trac via ajax.
    """
    trac = re.search('/trac/(?P<trac>[a-zA-z0-9\-_]+)',request.environ.get('HTTP_REFERER', ''))
    if trac:
        return {'trac':trac.group('trac')}
    else:
        return {}



# -*- coding: utf-8 -*-
import pyramid.tweens
from zope.interface import implements, Interface
from penelope.core.views import DefaultContext

DISCOVERY = False


def includeme(config):
    config.include('pyramid_skins')
    config.register_path("penelope.core.gdata:skins", discovery=DISCOVERY)

    config.add_view('penelope.core.gdata.views.clear_token', name='clear_gdata_token')
    config.add_view('penelope.core.gdata.views.add_token', name='add_gdata_token')
    config.add_view('penelope.core.gdata.views.generate_spreadsheet', name='generate_iteration', request_method="POST", permission='manage_iterations')
    config.add_view('penelope.core.gdata.views.activate_iteration', name='activate_iteration')
    config.add_tween('penelope.core.gdata.utils.fallback_tween_factory', over=pyramid.tweens.MAIN)

    config.add_route('view_iterations', '/view_iterations', factory='penelope.core.gdata.IterationContext')
    config.add_view('penelope.core.gdata.views.view_iterations', route_name='view_iterations', permission='view_iterations')

    config.add_route('manage_iterations', '/manage_iterations', factory='penelope.core.gdata.IterationContext')
    config.add_view('penelope.core.gdata.views.manage_iterations', route_name='manage_iterations', permission='manage_iterations')


class IIterationView(Interface):
    """Marker interface for iteration views"""


class IterationContext(DefaultContext):
    """Default context factory for Iteration views."""
    implements(IIterationView)




import mandrill

from pyramid.i18n import TranslationStringFactory
from pyramid.threadlocal import get_current_request
from pyramid.threadlocal import get_current_registry
from pyramid.view import view_config
from pyramid.response import Response

_ = TranslationStringFactory('penelope')


@view_config(route_name="inbound_email")
def inbound_email(request):
    return Response('OK')


def send(template_name, message):
    settings = get_current_registry().settings
    if not settings:
        return

    if not 'from_email' in message:
        message['from_email'] = settings['mail.default_sender']
        message['headers'] = {'Reply-To': settings['mail.default_sender']}

    mandrill_client = mandrill.Mandrill(settings['mail.password'])
    return mandrill_client.messages.send_template(template_name=template_name,
                                                  template_content=[],
                                                  message=message)


def notify_with_password_reset(email, token):
    request = get_current_request()
    if not request:
        return

    params = {"header": u'Password reset',
              "message": u'Please click on the link bellow to reset your penelope account\'s password.',
              "link": '%s/change_password?token=%s' % (request.application_url, token),
              "action": 'Reset password'}

    merged_params = []
    for k,v in params.items():
        merged_params.append({'name': k, 'content':v})

    message = {'auto_html': None,
               'auto_text': None,
               'from_name': 'RedTurtle Team',
               'important': True,
               'inline_css': True,
               'global_merge_vars': merged_params,
               'subject': u"Password reset request",
               'to': [{'email': email}]}

    return send('general', message)


def notify_user_with_welcoming_mail(email):
    request = get_current_request()
    if not request:
        return

    subject = _(u"Welcome to Penelope, Issue tracking system.")
    body = _(u"""Sei stato abilitato all'utilizzo di Penelope, la nostra piattaforma online di gestione dei progetti e dei "trouble ticket". Con Penelope potrai aprire nuovi ticket e seguire l'evoluzione delle tue segnalazioni. Ci raccomandiamo di verificare che i ticket che hai aperto abbiano la marcatura "Ticket Aperto dal Cliente = SI".
    <br/>
    Utilizzando questo link: %(url)s/password_reset_form?email=%(email)s potrai definire la tua nuova password di accesso a Penelope.
    RedTurtle ti ringrazia della collaborazione.
    <br/>
    <br/>
    You were enabled as a user of Penelope, our online projects and trouble ticket management platform. With penelope you will be able to open new tickets and follow the evolution of the issues you opened. We recommend to double check that the tickets you open have the "Ticked opened by customer" field set at "SI" (Yes).
    <br/>
    By this link: %(url)s/password_reset_form?email=%(email)s you can set your new Penelope password.
    RedTurtle thanks you for your collaboration.
    """ % {'email': email,
           'url' : request.application_url})

    params = {"header": subject,
              "message": body,
              "link": '%s/password_reset_form?email=%s' % (request.application_url, email),
              "action": 'Activate your account NOW!'}

    merged_params = []
    for k,v in params.items():
        merged_params.append({'name': k, 'content':v})

    message = {'auto_html': None,
               'auto_text': None,
               'from_name': 'RedTurtle Team',
               'important': True,
               'inline_css': True,
               'global_merge_vars': merged_params,
               'subject': subject,
               'to': [{'email': email}]}

    return send('general', message)



# -*- coding: utf-8 -*-
import re
import os
import httplib
import base64
try:
    import json
    json  # pyflakes
except ImportError:
    import simplejson as json

from robot.libraries.BuiltIn import BuiltIn

USERNAME_ACCESS_KEY = re.compile('^(http|https):\/\/([^:]+):([^@]+)@')


class SauceLabs:

    def report_sauce_status(self, name, status, tags=[], remote_url=''):
        """Report test status and tags to SauceLabs
        """
        job_id = BuiltIn().get_library_instance(
            'Selenium2Library')._current_browser().session_id

        if USERNAME_ACCESS_KEY.match(remote_url):
            username, access_key =\
                USERNAME_ACCESS_KEY.findall(remote_url)[0][1:]
        else:
            username = os.environ.get('SAUCE_USERNAME')
            access_key = os.environ.get('SAUCE_ACCESS_KEY')

        if not job_id:
            return u"No Sauce job id found. Skipping..."
        elif not username or not access_key:
            return u"No Sauce environment variables found. Skipping..."

        token = base64.encodestring('%s:%s' % (username, access_key))[:-1]
        body = json.dumps({'name': name,
                           'passed': status == 'PASS',
                           'tags': tags})

        connection = httplib.HTTPConnection('saucelabs.com')
        connection.request('PUT', '/rest/v1/%s/jobs/%s' % (
            username, job_id), body,
            headers={'Authorization': 'Basic %s' % token}
        )
        return connection.getresponse().status



import jsonrpc.proxy

from pyramid import testing
from webtest import TestApp

from penelope.core.models.dashboard import User, Project, Group

User.__mapper__.order_by = []
Project.manager.property.order_by = []
Group.users.property.order_by = []


def setUp(test, *args, **kwargs):
    testing.setUp()

def tearDown(test):
    testing.tearDown()

def Test_RPC_App(**kwargs):
    from penelope.core import main
    settings = {'test': True,
                'sa.dashboard.url': 'sqlite://',
                'velruse.openid.store':'openid.store.memstore.MemoryStore',
                'velruse.openid.realm':'localhost',
                }
    app = main({}, **settings)
    return TestApp(app, **kwargs)

app = Test_RPC_App()

def fakepost(self, url, data):
    if url.endswith('/'):
        url = url[:-1]
    respdata = app.post(url, data)
    respdata.read = lambda : respdata.body
    return respdata

jsonrpc.proxy.JSONRPCProxy._post = fakepost



# -*- coding: utf-8 -*-

import datetime
import unittest2

from penelope.core.smartadd import SmartAddParser


class TestSmartAddParser(unittest2.TestCase):

    def setUp(self):
        pass

    def test_plain(self):
        """
        A simple text with no tags
        """
        par = SmartAddParser(u'foo bar baz')
        self.assertEquals(par.unparsed, u'foo bar baz')
        self.assertIsNone(par.project_id)


    def test_strip(self):
        """
        Description has no leading or trailing spaces
        """
        par = SmartAddParser(u'  foo bar baz ')
        self.assertEquals(par.unparsed, u'foo bar baz')
        self.assertIsNone(par.project_id)

    #
    # PROJECT
    #
    # XXX what if the @ is used more than once? error or parse the longest found
    #     now it only gives an error if no tag matches

    def test_project(self):
        """
        A project tag that matches an existing project
        """
        projects = {'foo': '37', 'bar': '2', 'baz': '3'}
        par = SmartAddParser(u'this is some @foo stuff', projects=projects)
        self.assertEquals(par.unparsed, u'this is some stuff')
        self.assertEquals(par.project_id, '37')


    def test_project_ignorecase(self):
        """
        A project name is found if the case is different
        """
        projects = {'FOO': '37', 'bar': '2', 'baz': '3'}
        par = SmartAddParser(u'this is some @Foo stuff ^1', projects=projects)
        self.assertEqual([u'Ticket is missing'], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'this is some stuff',
            'project_id': '37',
            }, par.parsed_time_entry)


    def test_project_notfound(self):
        """
        A project control character is used but the project is not found
        """
        par = SmartAddParser(u'this is some @foo stuff #1 ^:30',
                             projects={'bar': '2', 'baz': '3'},
                             available_tickets=[1])
        self.assertEqual([u'Project not found'],
                          par.validation_errors())


    def test_project_first(self):
        """
        Only matches the first tag
        """
        projects = {'foo': '37', 'barbaz': '2'}
        par = SmartAddParser(u'this is some @foo @barbaz stuff ^1',
                             projects=projects)
        self.assertEqual([u'Ticket is missing'], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'this is some @barbaz stuff',
            'project_id': '37',
            }, par.parsed_time_entry)


    def test_project_longest(self):
        """
        Matches the longest project name, if there are several matches
        """
        projects = {'foo': '37', 'foobar': '2', 'foobaz': '3', 'fooblabla': '5'}
        par = SmartAddParser(u'this is some @fooblabla stuff ^1',
                             projects=projects)
        self.assertEqual([u'Ticket is missing'], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'this is some stuff',
            'project_id': '5',
            }, par.parsed_time_entry)


    def test_project_waitingfortickets(self):
        projects = {'foo': '37', 'foobar': '2', 'foobaz': '3', 'fooblabla': '5'}
        par = SmartAddParser(u'omg omg @fooblabla hey ^1 #', projects=projects)
        self.assertEqual([u'Ticket is missing'], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'omg omg hey #',
            'project_id': '5',
            }, par.parsed_time_entry)


    #
    # TICKET
    #

    def test_ticket(self):
        """
        A ticket tag that matches an existing ticket
        """
        tickets = [1, 2, 3, 4, 45]
        par = SmartAddParser(u'fixed #45 for ever ^1', available_tickets=tickets)
        self.assertEqual([u'Project is missing'], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'fixed for ever',
            'tickets': ['45'],
            }, par.parsed_time_entry)

    def test_ticket_notfound(self):
        """
        Missing tickets raise an error even when multiple tickets are selected.
        """
        par = SmartAddParser(u'@proj #3 #47 works for me ^1:00',
                             projects={'proj': 'proj'},
                             available_tickets=[1, 2, 3, 4, 45])
        self.assertEqual([u'Ticket #47 not found or customer request already invoiced'],
                          par.validation_errors())
        par = SmartAddParser(u'@proj #47 #3 #2 #66 works for me ^1:00',
                             projects={'proj': 'proj'},
                             available_tickets=[1, 2, 3, 4, 45])
        self.assertEqual([u'Tickets #47, #66 not found or customer request already invoiced'],
                          par.validation_errors())


    def test_ticket_callable(self):
        """
        A ticket list may be provided by a callable. The project_id is then passed to the callable.
        """
        def ticket_provider(project_code):
            if project_code == 'one':
                return [1]
            if project_code == 'two':
                return [2]
        par = SmartAddParser(u'@one #1 ^0:20 blabla',
                             projects={'one': 'one'},
                             available_tickets=ticket_provider)
        self.assertEqual([], par.validation_errors())

        par = SmartAddParser(u'@two #2 ^0:20 blabla',
                             projects={'two': 'two'},
                             available_tickets=ticket_provider)
        self.assertEqual([], par.validation_errors())

        par = SmartAddParser(u'@one #2 ^0:20 blabla',
                             projects={'one': 'one'},
                             available_tickets=ticket_provider)
        self.assertEqual([u'Ticket #2 not found or customer request already invoiced'],
                          par.validation_errors())

        par = SmartAddParser(u'@two #1 ^0:20 blabla',
                             projects={'two': 'two'},
                             available_tickets=ticket_provider)
        self.assertEqual([u'Ticket #1 not found or customer request already invoiced'],
                          par.validation_errors())


    def test_project_name(self):
        """
        The parser matches the project name, not the id.
        """
        par = SmartAddParser(u'@proj_name #47 works for me ^1:00',
                             projects={'proj_name': 'pr_id'},
                             available_tickets=[1, 2, 3, 4, 47])
        self.assertEqual([], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'works for me',
            'tickets': ['47'],
            }, par.parsed_time_entry)


    def test_project_name_uppercase(self):
        """
        The match is case insensitive.
        """
        par = SmartAddParser(u'@Proj_Name #47 works for me ^1:00',
                             projects={'Proj_Name': 'pr_id'},
                             available_tickets=[1, 2, 3, 4, 47])
        self.assertEqual([], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'works for me',
            'tickets': ['47'],
            }, par.parsed_time_entry)


    def test_project_name_space(self):
        """
        The project name can contain spaces.
        """
        par = SmartAddParser(u'@proj name #47 works for me ^1:00',
                             projects={'proj name': 'pr_id'},
                             available_tickets=[1, 2, 3, 4, 47])
        self.assertEqual([], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'works for me',
            'tickets': ['47'],
            }, par.parsed_time_entry)



    def test_project_with_space(self):
        """
        A project name may contain spaces (regression #85)
        """
        par = SmartAddParser(u'@my project #47 works for me ^1:00',
                             projects={'my project': 'proj'},
                             available_tickets=[1, 2, 3, 4, 47])
        self.assertEqual([], par.validation_errors())
        self.assertDictContainsSubset({
            'description': u'works for me',
            'tickets': ['47'],
            }, par.parsed_time_entry)


    def test_multi_tickets(self):
        """
        More than one ticket can be specified. Duration will be equally split.
        """
        tickets = [1, 2, 3, 4, 6, 10, 45, 66, 100, 666]
        projects = {'my project': 7}
        par = SmartAddParser(u'@my project fixed #6 #66 #45 lorem ipsum ^1:30',
                             projects=projects,
                             available_tickets=tickets)
        self.assertEquals(par.unparsed, u'fixed lorem ipsum')
        self.assertDictContainsSubset({
            'project_id': 7,
            'description': u'fixed lorem ipsum',
            'tickets': ['6', '66', '45'],
            'hours': datetime.timedelta(0, 5400),
            }, par.parsed_time_entry)



    def test_repeated_multi_tickets(self):
        """
        Tickets that are repeated only count for one.
        """
        tickets = [1, 2, 3, 4, 6, 10, 45, 66, 100, 666]
        projects = {'my project': 7}
        par = SmartAddParser(u'@my project fixed #6 #66 #45 #66 #45 lorem ipsum ^1:30',
                             projects=projects,
                             available_tickets=tickets)
        self.assertEquals(par.unparsed, u'fixed lorem ipsum')
        self.assertDictContainsSubset({
            'project_id': 7,
            'description': u'fixed lorem ipsum',
            'tickets': ['6', '66', '45'],
            'hours': datetime.timedelta(0, 5400),
            }, par.parsed_time_entry)




    #
    # HOURS
    #
    # XXX what if the ^ is used more than once? error?

    def test_hours(self):
        """
        Looks for a time like ^HH:MM
        """
        par = SmartAddParser(u'Took half an hour ^0:30.')
        self.assertDictContainsSubset({
            'description': u'Took half an hour .',
            'hours': datetime.timedelta(minutes=30),
            }, par.parsed_time_entry)


    def test_hours_integer(self):
        """
        ^3 is short for ^03:00
        """
        par = SmartAddParser(u'Took two ^2 hours')
        self.assertDictContainsSubset({
            'description': u'Took two hours',
            'hours': datetime.timedelta(hours=2),
            }, par.parsed_time_entry)


    def test_hours_onlyminutes(self):
        """
        ^:20 is short for ^00:20
        """
        par = SmartAddParser(u'Took thirty ^:30 minutes')
        self.assertDictContainsSubset({
            'description': u'Took thirty minutes',
            'hours': datetime.timedelta(minutes=30),
            }, par.parsed_time_entry)


    def test_hours_error(self):
        """
        A time control character is used but the time could not be parsed
        """
        par = SmartAddParser(u'@foo #3 this is^wrong',
                             projects={'foo':'foo'},
                             available_tickets=[3])
        self.assertEqual([u'Could not parse time'],
                          par.validation_errors())


    #
    # DATE
    #

    def test_dates(self):
        """
        A project tag that matches an existing project
        """
        projects = {'foo': '37', 'bar': '2', 'baz': '3'}
        par = SmartAddParser(u'!today this is some @foo stuff', projects=projects)
        self.assertEquals(par.unparsed, u'this is some stuff')
        self.assertEquals(par.date, datetime.date.today())

        par = SmartAddParser(u'this is !yesterday some @foo stuff', projects=projects)
        self.assertEquals(par.unparsed, u'this is some stuff')
        self.assertEquals(par.date, datetime.date.today() - datetime.timedelta(days=1))

        par = SmartAddParser(u'this is !foobar some @foo stuff #3 ^2:30',
                             projects=projects,
                             available_tickets=[3])

        self.assertEquals(par.unparsed, u'this is !foobar some stuff')
        self.assertEqual([u'Cannot parse date'],
                          par.validation_errors())

    #
    # Required tags
    #

    def test_complete(self):
        """
        Checks that all the required tags are present, and description is there
        """
        par = SmartAddParser(u'')
        self.assertEqual([u'Project is missing', u'Time is missing', u'Ticket is missing', u'Description is empty'],
                          par.validation_errors())

        par = SmartAddParser(u'bla bla bla')
        self.assertEqual([u'Project is missing', u'Time is missing', u'Ticket is missing'],
                          par.validation_errors())

        par = SmartAddParser(u'bla bla bla @ppp', projects={'ppp': 1})
        self.assertEqual([u'Time is missing', u'Ticket is missing'],
                          par.validation_errors())

        par = SmartAddParser(u'bla bla bla @ppp #2', projects={'ppp': 1}, available_tickets=[2])
        self.assertEqual([u'Time is missing'],
                          par.validation_errors())




if __name__ == '__main__':
    unittest2.main()




# -*- coding: utf-8 -*-

"""
Deform widgets for penelope.core
"""

from pkg_resources import resource_filename

import deform


class DeleteButton(deform.form.Button):
    icon_class = 'icon-trash'


class DownloadButton(deform.form.Button):
    icon_class = 'icon-white icon-download'


class FavoriteButton(deform.form.Button):
    icon_class = 'icon-white icon-star'


class RenameButton(deform.form.Button):
    icon_class = 'icon-white icon-edit'


class SearchButton(deform.form.Button):
    icon_class = 'icon-white icon-search'


class SubmitButton(deform.form.Button):
    icon_class = 'icon-white icon-ok'


class ResetButton(deform.form.Button):
    icon_class = 'icon-gray icon-remove'
    type = 'reset'


class PorInlineForm(deform.Form):
    default_renderer = deform.ZPTRendererFactory([
                            resource_filename('penelope.core', 'deform_templates'),
                            resource_filename('deform_bootstrap', 'templates'),
                            resource_filename('deform', 'templates'),
                            ])

    inline_form = True
    bootstrap_form_style = 'form-inline'
    css_class = u'well'


class WizardForm(deform.Form):
    default_renderer = deform.ZPTRendererFactory([
                            resource_filename('penelope.core', 'wizard_templates'),
                            resource_filename('penelope.core', 'deform_templates'),
                            resource_filename('deform_bootstrap', 'templates'),
                            resource_filename('deform', 'templates'),
                            ])

    inline_form = False
    bootstrap_form_style = 'form-inline'
    css_class = u'well'




import os
import sys
import beaker

from pyramid.paster import setup_logging, bootstrap

beaker.cache.cache_regions.update(dict(calculate_matrix={'key_length':''}))


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri>\n'
          '(example: "%s development.ini")' % (cmd, cmd)) 
    sys.exit(1)

def main(argv=sys.argv):
    if len(argv) != 2:
        usage(argv)

    config_uri = argv[1]
    setup_logging(config_uri)
    env = bootstrap('%s#dashboard'% config_uri)
    settings = env.get('registry').settings
    from penelope.core.scripts import populate_dummies
    populate_dummies(settings)



# -*- coding: utf-8 -*-

from copy import deepcopy
from fa.bootstrap import actions
from pyramid import httpexceptions as exc
from penelope.core.forms import ModelView
from penelope.core.forms import workflow


def configurate(config):
    config.formalchemy_model_view('admin',
                                  request_method='GET',
                                  permission='view',
                                  name='',
                                  attr='show',
                                  renderer='fa.bootstrap:templates/admin/show.pt',
                                  model='penelope.core.models.dashboard.Contract',
                                  view=ContractModelView)
    config.formalchemy_model_view('admin',
                                  request_method='GET',
                                  permission='workflow',
                                  name='goto_state',
                                  model='penelope.core.models.dashboard.Contract',
                                  view=workflow.goto_state)
    config.formalchemy_model_view('admin',
                                  request_method='POST',
                                  permission='delete',
                                  name='delete',
                                  attr='delete',
                                  renderer='fa.bootstrap:templates/admin/edit.pt',
                                  model='penelope.core.models.dashboard.Contract',
                                  view=ContractModelView)
    config.formalchemy_model_view('admin',
                                  renderer='penelope.core.forms:templates/contract_listing.pt',
                                  attr='datatable',
                                  context='pyramid_formalchemy.resources.ModelListing',
                                  request_method='GET',
                                  permission='listing',
                                  model='penelope.core.models.dashboard.Contract',
                                  view=ContractModelView)



contract_tabs = actions.TabsActions(actions.TabAction("show",
                                                      content="View",
                                                      permission='view',
                                                      attrs=dict(href="request.fa_url(request.model_name, request.model_id, '')")),)


class ContractModelView(ModelView):
    actions_categories = ('buttons', 'tabs')
    defaults_actions = deepcopy(actions.defaults_actions)

    def __init__(self, *args, **kwargs):
        super(ContractModelView, self).__init__(*args, **kwargs)
        self.defaults_actions.update(show_buttons=actions.Actions(actions.edit))
        cr_actions = deepcopy(contract_tabs)
        wf = workflow.change_workflow(self.context)
        if wf:
            cr_actions.append(wf)
        self.defaults_actions.update(show_tabs = cr_actions)

    def delete(self):
        cr = self.context.get_instance()
        if cr.customer_requests:
            request = self.request
            request.add_message(u'Contract has customer requests. Please remove them first.', type='danger')
            raise exc.HTTPFound(location=request.fa_url('Contract', cr.id))
        else:
            return self.force_delete()

    @actions.action('listing')
    def datatable(self, **kwargs):
        result = super(ContractModelView, self).datatable(**kwargs)
        fs = result['fs']
        fs.configure(pk=True, readonly=True)
        columns = ['name', 'project', 'amount', 'creation_date', 'contract_number', 'days', 'start_date',
                   'end_date', 'workflow_state', 'id']
        self.pick_columns(fs, columns)

        return dict(result,
                    columns=columns)



# -*- coding: utf-8 -*-

import os.path, time
import zope.component
from zope.interface import implements
from pyramid.view import view_config
from pyramid_formalchemy.resources import Base, actions as factions
from pyramid.renderers import get_renderer
from pyramid.threadlocal import get_current_registry
from penelope.core.forms import ModelView
from penelope.core.sidebar import SidebarRenderer, HeaderSidebarAction, SidebarAction
from penelope.core.interfaces import ISidebar
from penelope.core.interfaces import IManageView
from penelope.core.views import DefaultContext
from penelope.core.models import dashboard, DBSession
from penelope.core import forms


class ManageContext(DefaultContext):
    "Default context factory for Manage views."
    implements(IManageView)

    @property
    def title(self):
        view = self.request.view_name or self.request.environ['PATH_INFO'].split('/')[-1]
        if view == 'manage':
            view = 'index'
        return 'Manage %s' % self.prettify_title(view)


class ManageSidebarRenderer(SidebarRenderer):

    def render(self, request):
        self.actions.append(HeaderSidebarAction('manage',content=u'Manage penelope', permission='manage', no_link=True))
        self.actions.append(SidebarAction('manage_users',
                                          content=u'Manage users',
                                          permission='manage',
                                          attrs=dict(href="'%s/admin/User' % request.application_url")))
        self.actions.append(SidebarAction('manage_roles',
                                          content=u'Manage roles',
                                          permission='manage',
                                          attrs=dict(href="'%s/admin/Role' % request.application_url")))
        self.actions.append(SidebarAction('manage_groups',
                                          content=u'Manage groups',
                                          permission='manage',
                                          attrs=dict(href="'%s/admin/Group' % request.application_url")))
        self.actions.append(SidebarAction('manage_svn_authz',
                                          content=u'Manage SVN authz',
                                          permission='manage_svn',
                                          attrs=dict(href="'%s/manage/svn_authz' % request.application_url")))
        self.actions.append(SidebarAction('manage_costs',
                                          content=u'Manage company cost',
                                          permission='costs',
                                          attrs=dict(href="'%s/manage/costs' % request.application_url")))
        actions = self.actions.render(request)
        template =  get_renderer('penelope.core.forms:templates/project_sidebar.pt').implementation()
        return template(actions=actions,
                        request=request)

gsm = zope.component.getGlobalSiteManager()
gsm.registerAdapter(ManageSidebarRenderer, (IManageView,), ISidebar)


@view_config(route_name='administrator', permission='manage', renderer='skin')
def manage_home(request):
    return {}


@view_config(route_name='manage_svn_authz', renderer='skin', permission='manage_svn')
def manage_svn_authz(request):
    settings = get_current_registry().settings
    authz_file = settings.get('por.svn.authz')

    if request.method == 'POST':
        from penelope.trac.authz import generate_authz
        generate_authz(request.registry.settings)

    with open(authz_file, 'r') as configfile:
        authz = configfile.read()

    modified = time.ctime(os.path.getmtime(authz_file))
    return {'authz': authz,
            'authz_modified': modified}


@view_config(route_name='manage_costs', permission='manage', renderer='skin')
def manage_costs(context, request):

    # make fake FA request
    request.model_name = 'Company costs'
    request.model_class = dashboard.Cost
    request.session_factory = DBSession
    request.forms = forms
    request.format = 'html'
    request.actions = factions.RequestActions()
    request.actions['buttons'] = forms.actions.defaults_actions['new_buttons']
    request.action = 'add_cost'
    request.model_id = None
    request.route_name = 'manage_costs'
    base = Base(request, request.model_name)
    request.fa_url = base._fa_url

    global_config = DBSession().query(dashboard.GlobalConfig).first()
    view = ModelView(context, request)
    response = view.new()

    if request.method == 'GET':
        page = view.get_page(collection=global_config.costs)
        pager = page.pager(**view.pager_args)
        response['items'] = page
        response['pager'] = pager
    return response



# -*- coding: utf-8 -*-
from zope.interface import implements

from pyramid_formalchemy import resources
from penelope.core.interfaces import IManageView


def configurate(config):
    #custom view for role
    config.formalchemy_model_view('admin',
        view='penelope.core.forms.ModelView',
        context=ModelListing,
        model='penelope.core.models.dashboard.Role',
        renderer='pyramid_formalchemy:templates/admin/listing.pt',
        attr='listing',
        request_method='GET',
        permission='listing')

class ModelListing(resources.ModelListing):
    implements(IManageView)





import transaction
import redis
import pretty

from datetime import datetime
from time import sleep
from socketio.namespace import BaseNamespace
from penelope.core.models.dashboard import Activity
from penelope.core.models.dbsession import DBSession


class FeedlyNamespace(BaseNamespace):

    def listener(self):
        r = redis.StrictRedis()
        r = r.pubsub()
        r.subscribe('penelope:notifications:%s' % self.session['user_id']) # feedly pubsub key

        for m in r.listen():
            if m['type'] == 'message':
                sleep(1) # wait for the commit; to be fixed
                count_unseen, activities = self._get_activities_()
                self.emit("activities", {'count_unseen':count_unseen,
                                         'activities': activities})

    def _get_activities_(self):
        session = DBSession()
        base_query = session.query(Activity)\
                            .filter_by(user_id=self.session['user_id'])
        count_unseen = base_query.filter_by(seen_at=None).count()
        activities = base_query.order_by(Activity.created_at.desc()).limit(5)

        response = []
        for activity in activities:
            response.append({'message': activity.message,
                             'unseen': activity.unseen,
                             'absolute_path': activity.absolute_path,
                             'created_by': activity.created_by,
                             'created_at': pretty.date(activity.created_at)})

        transaction.commit()
        return count_unseen, response

    def on_join(self, data):
        self.session['user_id'] = data['user_id']
        self.spawn(self.listener)
        count_unseen, activities = self._get_activities_()
        self.emit("activities", {'count_unseen':count_unseen,
                                 'activities': activities})

    def on_mark_all_seen(self, data):
        user_id = data['user_id']
        session = DBSession()
        unseen = session.query(Activity)\
                        .filter_by(user_id=user_id)\
                        .filter_by(seen_at=None)
        now = datetime.now()
        for a in unseen:
            a.seen_at = now
        transaction.commit()
        r = redis.StrictRedis()
        r.publish('penelope:notifications:%s' % user_id, True)

    def recv_disconnect(self):
        self.disconnect(silent=True)


def add_activity(user_ids, message, absolute_path, created_by=None):
    if created_by is None:
        created_by = 'penelopedev@redturtle.it'

    r = redis.StrictRedis()
    session = DBSession()
    for user_id in user_ids:
        activity = Activity(message=message,
                            created_at=datetime.now(),
                            user_id=user_id,
                            absolute_path=absolute_path,
                            created_by=created_by)

        session.add(activity)
        r.publish('penelope:notifications:%s' % user_id, True)
    transaction.commit()



# -*- coding: utf-8 -*-

import json
import logging
import colander

from sqlalchemy import distinct

from deform import ValidationFailure
from pyramid.view import view_config
from webhelpers.html.builder import literal

from penelope.core.models import (
    DBSession,
    Project,
    Customer,
    Contract,
    CustomerRequest
)
from penelope.core import fanstatic_resources
from penelope.core.lib.widgets import SearchButton, PorInlineForm
from penelope.core.reports.validators import validate_period
from penelope.core.reports.queries import (
    qry_active_projects,
)
from penelope.core.reports import fields

log = logging.getLogger(__name__)


class IDTree(dict):
    # XXX see if we can do without this silly tree

    def insert_entry(self, groupby, cr_id, customer,
                     project, request, contract, **ignore):
        """
        populates the tree with time-entry ids
        """
        node = self
        for idx, nodetype in enumerate(groupby, 1):
            if idx < len(groupby):
                default = {}
            else:
                default = []

            if nodetype == 'customer':
                node.setdefault(customer, default)
                node = node[customer]
            elif nodetype == 'project':
                node.setdefault(project, default)
                node = node[project]
            elif nodetype == 'request':
                node.setdefault(request, default)
                node = node[request]
            elif nodetype == 'contract':
                node.setdefault(contract, default)
                node = node[contract]
            else:
                raise ValueError('unsupported groupby value?')
        node.append(cr_id)


class TekkenReport(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def format_xls(self, value):
        if isinstance(value, literal):
            # sanitize HTML
            return value.striptags()
        return value

    class TekkenSchema(colander.MappingSchema):
        customer_id = fields.customer_id.clone()
        project_id = fields.project_id.clone()
        date_from = fields.date_from.clone()
        date_to = fields.date_to.clone()

    def search(self, customer_id, project_id, date_from, date_to):
        qry = DBSession.query(CustomerRequest)\
                       .join(Contract)\
                       .join(Project)\
                       .join(Customer)\
                       .filter(Project.activated == True)  # noqa

        groupby = ['customer', 'project', 'contract', 'request']

        if customer_id is not colander.null:
            qry = qry.filter(Project.customer_id == customer_id)

        if project_id is not colander.null:
            qry = qry.filter(Project.id == project_id)

        if date_from is not colander.null:
            qry = qry.filter(Contract.end_date >= date_from)

        if date_to is not colander.null:
            qry = qry.filter(Contract.start_date <= date_to)

        rows = []
        id_tree = IDTree()

        for cr in qry:
            entry = {
                'customer': cr.project.customer.name.strip(),
                'project': cr.project.name.strip(),
                'request': cr.name.strip(),
                'contract': cr.contract.name.strip(),
                'estimated_days_label': 'Estimated days',
                'estimated_days': cr.estimation_days,
                'worked_days_label': 'Worked days',
                'worked_days': cr.timeentries_days
            }
            rows.append(entry)
            id_tree.insert_entry(groupby, cr.id, **entry)

        return {
                'groupby': groupby,
                'rows': rows,
                'id_tree': id_tree,
                }

    @view_config(name='report_tekken', route_name='reports',
                 renderer='skin', permission='report_tekken')
    def __call__(self):
        fanstatic_resources.report_tekken.need()
        schema = self.TekkenSchema(validator=validate_period).clone()

        projects = qry_active_projects()
        project_ids = [p.id for p in projects]

        c = [distinct(Customer.id), Customer.id, Customer.name]
        customers = DBSession.query(*c)\
                             .join(Project)\
                             .filter(Project.id.in_(project_ids))\
                             .order_by(Customer.name)

        form = PorInlineForm(schema,
                             action=self.request.current_route_url(),
                             formid='all_entries',
                             method='GET',
                             buttons=[
                                 SearchButton(title=u'Search'),
                             ])

        form['customer_id'].widget.values = [('', '')] + \
            [(str(c.id), c.name) for c in customers]

        form['project_id'].widget.values = [('', '')] + \
            [(str(p.id), p.name) for p in projects]

        controls = self.request.GET.items()
        base_link = self.request.path_url.rsplit('/', 1)[0]
        xls_link = ''.join(
            [base_link, '/', 'tekken_xls', '?', self.request.query_string]
        )

        if not controls:
            # the form is empty
            return {
                    'form': form.render(),
                    'xls_link': xls_link,
                    'qs': '',
                    'has_results': False
                    }

        try:
            appstruct = form.validate(controls)
        except ValidationFailure as e:
            return {
                    'form': e.render(),
                    'xls_link': xls_link,
                    'qs': self.request.query_string,
                    'has_results': False
                    }

        entries_detail = self.search(**appstruct)

        col_headers = {
            'customer': u'Customer',
            'project': u'Project',
            'contract': u'Contract',
            'request': u'Request',
        }

        columns = [
            {'colvalue': 'estimated_days_label',
             'groupbyrank': None,
             'pivot': True,
             'result': False
             },
            {'colvalue': 'worked_days_label',
             'groupbyrank': None,
             'pivot': True,
             'result': False
             }
        ]

        for idx, colname in enumerate(entries_detail['groupby']):
            columns.append({
                            'colvalue': colname,
                            'coltext': colname,
                            'header': col_headers[colname],
                            'groupbyrank': idx+1,
                            'pivot': False,
                            'result': False
                           })

        columns.append({'colvalue': 'estimated_days',
                        'groupbyrank': None,
                        'pivot': False,
                        'result': True})
        columns.append({'colvalue': 'worked_days',
                        'groupbyrank': None,
                        'pivot': False,
                        'result': True})

        sourcetable = {
                    'rows': entries_detail['rows'],
                    'columns': columns
                }

        return {
                'form': form.render(appstruct=appstruct),
                'qs': self.request.query_string,
                'xls_link': xls_link,
                'has_results': len(sourcetable['rows']) > 0,
                'tpReport_oConf': json.dumps({
                                    'sourcetable': sourcetable,
                                    'id_tree': entries_detail['id_tree'],
                                    'groupby': entries_detail['groupby'],
                                }),
                }

    @view_config(name='tekken_xls', route_name='reports',
                 renderer='xls_report', permission='report_tekken')
    def tekken_xls(self):
        schema = self.TekkenSchema(validator=validate_period).clone()
        form = PorInlineForm(schema)
        controls = self.request.GET.items()
        appstruct = form.validate(controls)

        detail = self.search(**appstruct)
        columns = [('project', 'Project'),
                   ('customer', 'Customer'),
                   ('contract', 'Contract'),
                   ('request', 'CR'),
                   ('estimated_days', 'Estimated days'),
                   ('worked_days', 'Worked days')]

        rows = [
                [
                    self.format_xls(row[col_key])
                    for col_key, col_title in columns
                    ]
                for row in detail['rows']
                ]

        return {
                'header': [col_title for col_key, col_title in columns],
                'rows': rows,
                }



# -*- coding: utf-8 -*-
import collections
from zope.interface import implements
from copy import deepcopy
from datetime import datetime, timedelta

from pyramid import httpexceptions as exc
from pyramid_formalchemy import resources
from fa.bootstrap import actions

from penelope.core.fanstatic_resources import user_stats
from penelope.core.forms import ModelView
from penelope.core.forms.renderers import ProjectRelationRenderer
from penelope.core.interfaces import IManageView
from penelope.core.models import dashboard, tp, DBSession
from penelope.core.notifications import notify_user_with_welcoming_mail
from penelope.core.lib.helpers import timedelta_as_work_hours


def configurate(config):
    #custom view for user
    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='view',
        name='',
        attr='show',
        renderer='penelope.core.forms:templates/user.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='POST',
        permission='delete',
        name='delete',
        attr='delete',
        renderer='fa.bootstrap:templates/admin/edit.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        renderer='penelope.core.forms:templates/user_listing.pt',
        attr='datatable',
        context=ModelListing,
        request_method='GET',
        permission='listing',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    #custom view for tokens section
    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='edit',
        name='user_tokens',
        attr='user_tokens',
        renderer='penelope.core.forms:templates/user_tokens.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='costs',
        name='user_costs',
        attr='user_costs',
        renderer='penelope.core.forms:templates/user_costs.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='stats',
        name='user_stats',
        attr='user_stats',
        renderer='penelope.core.forms:templates/user_stats.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='stats',
        name='user_stats.json',
        attr='json_user_stats',
        renderer='json',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='costs',
        name='send_user_invitation',
        attr='send_user_invitation',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    #custom view for adding a customer request to the project
    config.formalchemy_model_view('admin',
        request_method='GET',
        permission='costs',
        name='add_cost',
        attr='add_cost',
        renderer='penelope.core.forms:templates/new.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)

    config.formalchemy_model_view('admin',
        request_method='POST',
        permission='costs',
        name='add_cost',
        attr='add_cost',
        renderer='penelope.core.forms:templates/new.pt',
        model='penelope.core.models.dashboard.User',
        view=UserModelView)


send_user_invitation = actions.UIButton(id='send_user_invitation',
    content='Send user invitation',
    permission='costs',
    _class='btn btn-success',
    attrs=dict(href="request.fa_url('User', request.model_id, 'send_user_invitation')"))


user_tabs = actions.Actions(
        actions.TabAction("show",
            content="View",
            permission='view',
            attrs=dict(href="request.fa_url(request.model_name, request.model_id, '')")),
        actions.TabAction("user_tokens",
            content="User tokens",
            permission='edit',
            attrs=dict(href="request.fa_url(request.model_name, request.model_id, 'user_tokens')")),
        actions.TabAction("costs",
            content="User costs",
            permission='costs',
            attrs=dict(href="request.fa_url(request.model_name, request.model_id, 'user_costs')")),
        actions.TabAction("stats",
            content="User stats",
            permission='stats',
            attrs=dict(href="request.fa_url(request.model_name, request.model_id, 'user_stats')")),
    )

add_costs = actions.Actions(
        actions.UIButton(id='add_cost',
            content='Add user cost',
            permission='costs',
            _class='btn btn-success',
            attrs=dict(href="request.fa_url('User', request.model_id, 'add_cost')")),
        )

class ModelListing(resources.ModelListing):
    implements(IManageView)


class UserModelView(ModelView):
    actions_categories = ('buttons', 'tabs')

    defaults_actions = deepcopy(actions.defaults_actions)
    defaults_actions['show_buttons'].append(send_user_invitation)
    defaults_actions.update(show_tabs=user_tabs)
    defaults_actions.update(user_costs_tabs=user_tabs)
    defaults_actions.update(user_costs_buttons=add_costs)
    defaults_actions.update(user_stats_tabs=user_tabs)

    @actions.action('show')
    def user_tokens(self, *args, **kwargs):
        context = self.context.get_instance()
        return self.render(user=context)

    @actions.action('user_costs')
    def user_costs(self, *args, **kwargs):
        context = self.context.get_instance()
        page = self.get_page(collection=context.costs)
        pager = page.pager(**self.pager_args)
        return self.render(pager=pager, items=page)

    @actions.action('user_costs')
    def add_cost(self, *args, **kwargs):
        self.request.model_class = dashboard.Cost
        self.request.model_name = dashboard.Cost.__name__
        self.request.form_action = ['User', self.request.model_id, 'add_cost']
        if self.request.method == 'POST':
            self.request.POST.update({'next': self.request.fa_url('User', self.request.model_id, 'user_costs')})
        return self.new()

    @actions.action('listing')
    def datatable(self, **kwargs):
        result = super(UserModelView, self).datatable(**kwargs)

        fs = result['fs']
        fs.configure(pk=False, readonly=True)

        columns = ['email', 'fullname', 'roles', 'project_manager', 'active']
        self.pick_columns(fs, columns)

        fs._render_fields['project_manager']._get_renderer = lambda: ProjectRelationRenderer(fs._render_fields['project_manager'])

        return dict(result,
                    columns=columns)

    def delete(self):
        user = self.context.get_instance()
        request = self.request
        if not user.active:
            request.add_message(u'User is already inactive. You cannot remove it.', type='danger')
            raise exc.HTTPFound(location=request.fa_url('User', user.id))
        else:
            user.active = False
            request.add_message(u'User deactivated.', type='success')
            raise exc.HTTPFound(location=request.fa_url('User', user.id))

    def send_user_invitation(self):
        user = self.context.get_instance()
        notify_user_with_welcoming_mail(user.email)
        request = self.request
        request.add_message(u'Invitation email send.', type='success')
        raise exc.HTTPFound(location=request.fa_url('User', user.id))

    @actions.action('user_stats')
    def user_stats(self, *args, **kwargs):
        context = self.context.get_instance()
        user_stats.need()
        return self.render(user=context)

    def json_user_stats(self, *args, **kwargs):

        user = self.context.get_instance()
        now = datetime.now().date()
        week_ago = now - timedelta(weeks=1)

        def find_tickets():
            all_tracs = DBSession.query(dashboard.Trac).join(dashboard.Project).filter(dashboard.Project.active)
            if all_tracs.count() == 0:
                return []
            query = """SELECT DISTINCT '%(trac)s' AS trac_name,
                                       TO_CHAR(to_timestamp(ticket.changetime/1000000),'YYYY-MM-DD') as modification,
                                       COUNT(DISTINCT ticket.id)
                                       FROM "trac_%(trac)s".ticket AS ticket
                                       LEFT OUTER JOIN "trac_%(trac)s".ticket_change AS change ON ticket.id=change.ticket
                                       WHERE change.author = '%(owner)s' AND to_timestamp(ticket.changetime/1000000) >= current_date - interval '6 days'
                                       GROUP BY modification
                                    """

            queries = []
            for trac in all_tracs:
                queries.append(query % {'trac': trac.trac_name,
                                        'owner': user.email})
            sql = '\nUNION '.join(queries)
            sql += ';'

            tracs =  DBSession().execute(sql).fetchall()
            return tracs

        colors = ["#637b85", "#2c9c69", "#dbba34", "#c62f29", "#F7464A",]
        project_hours = collections.defaultdict(float)
        dayofweek_hours = collections.defaultdict(float)

        tp_last_week = DBSession().query(tp.TimeEntry)\
                                  .filter(tp.TimeEntry.author_id==user.id)\
                                  .filter(tp.TimeEntry.date > week_ago)\
                                  .filter(tp.TimeEntry.date <= now)
        for t in tp_last_week:
            dayofweek_hours[t.date] += timedelta_as_work_hours(t.hours)
            project_hours[t.project.name] += t.hours_as_work_days

        project_hours = project_hours.items()
        project_hours.sort(key=lambda t: t[1], reverse=True)
        project_hours = project_hours[:5]

        last_week_hours = [{'value': p[1], 'color':colors[n]} for n,p in enumerate(project_hours)]
        last_week_hours_legend = [{'label': p[0], 'value': p[1]} for p in project_hours]

        tickets = find_tickets()

        w1 = set([a for a in dayofweek_hours.keys()])
        w2 = set([datetime.strptime(a[1], '%Y-%m-%d').date() for a in tickets])
        daysofweek = list(w1.union(w2))
        daysofweek.sort()

        project_tickets = collections.defaultdict(int)
        dayofweek_tickets = collections.defaultdict(int)
        for t in tickets:
            dayofweek_tickets[datetime.strptime(t[1], '%Y-%m-%d').date()] += t[2]
            project_tickets[t[0]] += t[2]

        project_tickets = project_tickets.items()
        project_tickets = project_tickets[:5]
        last_week_tickets = [{'value': p[1], 'color':colors[n]} for n,p in enumerate(project_tickets)]
        last_week_tickets_legend = [{'label': p[0], 'value': p[1]} for p in project_tickets]

        last_week_dow = {
                'labels' : [a.strftime('%A') for a in daysofweek],
                'datasets' : [
                    {
                        'fillColor' : "rgba(151,187,205,0.5)",
                        'strokeColor' : "rgba(151,187,205,1)",
                        'pointColor' : "rgba(151,187,205,1)",
                        'pointStrokeColor' : "#fff",
                        'data' : [dayofweek_hours.get(a,0) for a in daysofweek]
                        },
                    {
                        'fillColor' : "rgba(220,220,220,0.5)",
                        'strokeColor' : "rgba(220,220,220,1)",
                        'pointColor' : "rgba(220,220,220,1)",
                        'pointStrokeColor' : "#fff",
                        'data' : [dayofweek_tickets.get(a,0) for a in daysofweek]
                        },
                    ]
                }


        return {'last_week_hours': last_week_hours,
                'last_week_hours_legend': last_week_hours_legend,
                'last_week_tickets': last_week_tickets,
                'last_week_tickets_legend': last_week_tickets_legend,
                'last_week_dow': last_week_dow}



# -*- coding: utf-8 -*-
import os

PORT = os.environ.get('APP_PORT', 8080)
SELENIUM_IMPLICIT_WAIT = os.environ.get('SELENIUM_IMPLICIT_WAIT', '0.1s')
SELENIUM_TIMEOUT = os.environ.get('SELENIUM_IMPLICIT_WAIT', '20s')
USERNAME = 'admin@example.com'
PASSWORD = 'admin@example.com'
BUILD_NUMBER = os.environ.get('BUILD_NUMBER', 'manual')
SELENIUM_VERSION = '2.39.0'


APP_HOST = os.environ.get('APP_HOST', "localhost")
APP_URL = os.environ.get('APP_URL', "http://%s:%s" % (APP_HOST, PORT))
BROWSER = os.environ.get('BROWSER', "Firefox")
REMOTE_URL = os.environ.get('REMOTE_URL', "")
DESIRED_CAPABILITIES = os.environ.get('DESIRED_CAPABILITIES', "")



# -*- coding: utf-8 -*-

import pkg_resources

from fa.bootstrap.fanstatic_resources import bootstrap
from fanstatic import Group, Library, Resource
from js.bootstrap import bootstrap_js, bootstrap_responsive_css
from js.chart import chart_js
from js.jquery import jquery
from js.jquery_datatables import jquery_datatables_js    # jquery_datatables_css only contains a style for the demo table.
from js.jquery_timepicker_addon import timepicker_it
from js.jqueryui import jqueryui, jqueryui_i18n
from js.jqueryui import overcast
from js.socketio import socketio
from js.xeditable import bootstrap_editable


class ForeignLibrary(Library):
    """
    Let fanstatic get resources from an egg != the current one.
    Note that the library name must also be the package name, and must
    still be defined in the entry points.
    """
    def __init__(self, *args, **kw):
        super(ForeignLibrary, self).__init__(*args, **kw)
        self.path = pkg_resources.resource_filename(self.name, self.rootpath)



#----------------------
# Third party libraries
#----------------------


por_library = Library('por', 'static')

angular = Resource(por_library, 'angular/angular.js', depends=[jqueryui], minified='angular/angular.min.js')
angular_animation = Resource(por_library, 'angular/angular-animate.min.js', depends=[angular])
angular_dd = Resource(por_library, 'angular/angular-dragdrop.js', minified='angular/angular-dragdrop.min.js', depends=[jqueryui, angular])
angular_bootstrap_ui = Resource(por_library, 'angular/ui-bootstrap-tpls-0.5.0.min.js', depends=[bootstrap_js, angular])
angular_sortable = Resource(por_library, 'angular/sortable.js', depends=[jqueryui, angular])
angles = Resource(por_library, 'angular/angles.js', depends=[angular, chart_js])
font_awesome = Resource(por_library, 'font_awesome/css/font-awesome.css', minified='font_awesome/css/font-awesome.min.css', depends=[bootstrap_responsive_css])
js_md5 = Resource(por_library, 'md5/md5.js', minified='md5/md5.min.js',)
jquery_pivot_css = Resource(por_library, 'jquery.pivot/jquery_pivot.css')
jquery_pivot_costs = Resource(por_library, 'jquery.pivot/jquery_pivot_costs.js', depends=[jquery, jquery_pivot_css])
jquery_pivot_tekken = Resource(por_library, 'jquery.pivot/jquery_pivot_tekken.js', depends=[jquery, jquery_pivot_css])
jquery_pivot = Resource(por_library, 'jquery.pivot/jquery_pivot.js', depends=[jquery, jquery_pivot_css])
jquery_pnotify_default_css = Resource(por_library, 'jquery.pnotify/jquery.pnotify.default.css')
jquery_pnotify_icons_css = Resource(por_library, 'jquery.pnotify/jquery.pnotify.default.icons.css')
jquery_pnotify_js = Resource(por_library, 'jquery.pnotify/jquery.pnotify.js', minified='jquery.pnotify/jquery.pnotify.min.js', depends=[jqueryui])
pnotify = Group([jquery_pnotify_default_css, jquery_pnotify_icons_css, jquery_pnotify_js])
jsonrpc = Resource(por_library, 'jquery.jsonrpc/jquery.jsonrpc.js')
mustache = Resource(por_library, 'js.mustache/mustache.js', minified='js.mustache/mustache.min.js')
respond = Resource(por_library, 'js.respond/respond.src.js', minified='js.respond/respond.min.js')
spark = Resource(por_library, 'jquery.sparkline/jquery.sparkline.min.js', depends=[jquery,])
spin = Resource(por_library, 'js.spin/spin.js')


#-------
# Deform
#-------

deform_library = ForeignLibrary('deform', 'static')
deform_bootstrap_library = ForeignLibrary('deform_bootstrap', 'static')

deform_js = Resource(deform_library, 'scripts/deform.js', depends=[jquery])
deform_bootstrap = Group([
        Resource(deform_bootstrap_library, 'chosen_bootstrap.css'),
        Resource(deform_bootstrap_library, 'deform_bootstrap.js', depends=[deform_js]),
        Resource(por_library, 'jquery.chosen/chosen.min.css'),
        Resource(por_library, 'jquery.chosen/chosen.jquery.min.js', depends=[jquery]),
        Resource(deform_library, 'scripts/jquery.form.js', depends=[jquery]),
        ])


###########
# REPORTS #
###########

project_filter = Resource(por_library, 'por_report/select_project_filter.js')

subnav = Group([
                Resource(por_library, 'por_subnav/subnav.js', depends=[jquery]),
                Resource(por_library, 'por_subnav/subnav.css'),
               ])

all_entries = Group([
    Resource(por_library, 'por_report/all_entries.js', depends=[jquery, timepicker_it]),
    Resource(por_library, 'por_report/all_entries.css'),
    ])

report_all_entries = Group([
    jquery_pivot,
    all_entries
    ])

report_costs = Group([
    jquery_pivot_costs,
    all_entries
    ])

report_tekken = Group([
    jquery_pivot_tekken,
    all_entries
    ])

saved_queries = Resource(por_library, 'por_report/saved_queries.js', depends=[jquery])

report_te_state_change = Group([
                                Resource(por_library, 'por_report/te_state_change.js', depends=[jquery, timepicker_it]),
                                Resource(por_library, 'por_report/te_state_change.css'),
                                ])
#----------
# Our stuff
#----------

dashboard_home = Group([
                        Resource(por_library, 'por_home/js/home.js', depends=[spark]),
                        Resource(por_library, 'por_home/css/home.css'),
#                        Resource(por_library, 'por_home/js/outstanding_tickets.js', depends=[jquery, mustache, spin]),
                        Resource(por_library, 'por_home/css/penelope.min.css'),
                        ])


add_entry_from_ticket = Group([
                                Resource(por_library, 'por_add_entry/add_entry_from_ticket.js', depends=[jquery, mustache, timepicker_it, jsonrpc]),
                                Resource(por_library, 'por_add_entry/add_entry_from_ticket.css'),
                                ])

wizard = Group([ Resource(por_library, 'por_wizard/wizard.css') ])
fastticketing = Group([ Resource(por_library, 'por_fastticketing/fastticketing.css') ])
migrate_cr = Group([
                    Resource(por_library, 'por_migrate_cr/migrate.js', depends=[deform_bootstrap])])

add_entry = Group([
                    Resource(por_library, 'por_add_entry/add_entry.js', depends=[jquery]),
                    Resource(por_library, 'por_add_entry/add_entry.css'),
                    Resource(por_library, 'por_add_entry/autocomplete.js', depends=[jquery]),
                    spin,
                   ])

backlog = Group([
                    Resource(por_library, 'por_backlog/backlog.js', depends=[jquery]),
                    Resource(por_library, 'por_backlog/backlog.css'),
                    ])

datatables = Group([
                    Resource(por_library, 'por_datatables/js/sorting.js'),
                    Resource(por_library, 'por_datatables/js/paging.js', depends=[jquery_datatables_js]),
                    Resource(por_library, 'por_datatables/css/paging.css'),
                   ])

penelope_angular_js = Resource(por_library, 'penelope/js/penelope_angular.js',
        depends=[socketio, angular, js_md5, angular_animation, angular_bootstrap_ui, angular_sortable, bootstrap_editable, angles])


user_stats_css = Resource(por_library, 'por_userstats/userstats.css')
user_stats = Resource(por_library, 'por_userstats/userstats.js', depends=[penelope_angular_js, user_stats_css])

kanban = Group([
                Resource(por_library, 'por_kanban/js/kanban.js', depends=[bootstrap_js, penelope_angular_js, pnotify]),
                Resource(por_library, 'por_kanban/css/kanban.css', depends=[bootstrap]),
    ])

dashboard_js = Resource(por_library, 'penelope/js/dashboard.js', depends=[bootstrap_js, jqueryui, timepicker_it, penelope_angular_js])
dashboard_css = Resource(por_library, 'penelope/css/dashboard.css', depends=[bootstrap_responsive_css, respond, font_awesome])
dashboard = Group([jquery, deform_bootstrap, jqueryui, jqueryui_i18n, overcast, bootstrap, dashboard_js, dashboard_css, subnav])



from sqlalchemy import Column, Unicode
from zope.interface import implements
from repoze.workflow import get_workflow

from penelope.core.models.interfaces import IWorkflowEnabled


class Workflow(object):
    """
    Workflow enable implementation for POR project
    """
    implements(IWorkflowEnabled)

    workflow_state = Column(Unicode)

    def __init__(self, *args, **kwargs):
        super(Workflow, self).__init__(*args, **kwargs)
        wf = get_workflow(self, self.__class__.__name__)
        if wf:
            wf.initialize(self)



# -*- coding: utf-8 -*-
from penelope.core.forms import ModelView


def configurate(config):
    config.formalchemy_model_view('admin',
        request_method='POST',
        permission='delete',
        name='delete',
        attr='delete',
        renderer='fa.bootstrap:templates/admin/edit.pt',
        model='penelope.core.models.dashboard.Cost',
        view=CostModelView)


class CostModelView(ModelView):

    def delete(self):
        return self.force_delete()



from pyramid.response import Response
from pyramid.view import view_config
from socketio import socketio_manage

from penelope.core.activity_stream import FeedlyNamespace
from penelope.core.kanbanboard import KanbanNamespace


@view_config(route_name="socketio")
def socketio(request):
    socketio_manage(request.environ, {"/kanban": KanbanNamespace,
                                      "/feedly": FeedlyNamespace }, request=request)
    return Response('')



# -*- coding: utf-8 -*-
from pyramid.threadlocal import get_current_request
from fa.bootstrap import actions
from webob.exc import HTTPFound
from repoze.workflow import WorkflowError
from repoze.workflow import get_workflow


def goto_state(context, request):
    state = request.params.get('state')
    if not state:
        request.add_message(u'Missing state parameter.', 'error')
        return HTTPFound(location=request.fa_url(request.model_name, request.model_id))
    try:
        workflow = get_workflow(request.model_instance, request.model_name)
        workflow.transition_to_state(context.get_instance(), request, state, skip_same=False)
        request.add_message(u'Workflow status has been changed. New workflow state is: <strong>%s</strong>.' % state)
        return HTTPFound(location=request.fa_url(request.model_name, request.model_id))
    except WorkflowError, msg:
        request.add_message(msg, 'error')
        return HTTPFound(location=request.fa_url(request.model_name, request.model_id))


def change_workflow(context):
    try:
        wf = get_workflow(context.get_instance(), context.get_model().__name__)
    except TypeError:
        wf = None
    if not wf: return
    wf_actions = actions.DropdownActions(id='change_workflow',
            permission='workflow',
            content='%s state' % context.get_instance().workflow_state)

    request = get_current_request()
    states = wf.get_transitions(context.get_instance(), request)
    if not states:
        return None
    for state in states:
        attrs = {'href': "request.fa_url(request.model_name, request.model_id, 'goto_state', state='%s')" % state['to_state']}
        wf_actions.append(actions.TabAction(id=state['to_state'],
            permission='workflow',
            content=state['name'],
            attrs=attrs))
    return wf_actions


def validate_contract_done(content, info):
    """ raise WorkflowError when contract has active CustomerRequests. """
    if [a for a in content.customer_requests if a.active]:
        raise WorkflowError(u'Contract cannot be closed - there are customer requests still open.')


def validate_cr_unachieving(content, info):
    if content.contract and not content.contract.active and info.transition['name'] != 'estimating':
        raise WorkflowError(u'Workflow cannot be changed - related contract is closed.')
